/*
 * Copyright (c) 2005, Stefan Walter
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 * 
 *     * Redistributions of source code must retain the above 
 *       copyright notice, this list of conditions and the 
 *       following disclaimer.
 *     * Redistributions in binary form must reproduce the 
 *       above copyright notice, this list of conditions and 
 *       the following disclaimer in the documentation and/or 
 *       other materials provided with the distribution.
 *     * The names of contributors to this software may not be 
 *       used to endorse or promote products derived from this 
 *       software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
 * DAMAGE.
 * 
 *
 * CONTRIBUTORS
 *  Stefan Walter <stef@memberwebs.com>
 */ 
 
#ifndef __CONFIG_PARSER_H__
#define __CONFIG_PARSER_H__

#include <bsnmp/asn1.h>
#include <bsnmp/snmp.h>

/* Callbacks must be defined by the caller */
extern int cfg_value(const char* filename, const char* header, const char* name, 
                     char* value, void* data);
extern int cfg_error(const char* filename, const char* errmsg, void* data);

/* Calling these will call the callbacks above */
int cfg_parse_dir(const char* dirname, void* data);
int cfg_parse_file(const char* filename, void* data, char** memory);

/* A helper for parsing URIs */
const char* cfg_parse_uri (char *uri, char** scheme, char** host, char** user, char** path, char** query);

/* Parsing snmp, snmp2 snmp2c etc... */
const char* cfg_parse_scheme (const char *input, enum snmp_version *scheme);

/* Parse query=xxxx arguments */
const char* cfg_parse_query (char *query, char **name, char **value, char **remainder);

/* A helper for URL decoding URI values */
const char* cfg_parse_url_decode (char *value);

#endif /* __CONFIG_PARSER_H__ */
